@@ -107,7 +107,11 @@ ListSubtype_New( PyTypeObject* subtype, Py_ssize_t size )
             return PyErr_NoMemory();  // LCOV_EXCL_LINE
         memset( op->ob_item, 0, nbytes );
     }
+#if PY_VERSION_HEX >= 0x03090000
+    Py_SET_SIZE( op, size );
+#else
     Py_SIZE( op ) = size;
+#endif
     op->allocated = size;
     return ptr.release();
 }
@@ -298,6 +302,7 @@ int AtomList_traverse( AtomList* self, visitproc visit, void* arg )
     // This was not needed before Python 3.9 (Python issue 35810 and 40217)
     Py_VISIT(Py_TYPE(self));
 #endif
+    // PyList_type is not heap allocated so it does visit the type
 	return PyList_Type.tp_traverse( pyobject_cast( self ), visit, arg );
 }
 
@@ -688,10 +693,21 @@ public:
     PyObject* sort( PyObject* args, PyObject* kwargs )
     {
         static char *kwlist[] = { "key", "reverse", 0 };
-        cppy::ptr meth( PyObject_GetAttrString( m_list.get(), "sort") );
+        // Get a reference to builtins (borrowed ref hence the incref)
+        cppy::ptr builtins( cppy::incref( PyImport_AddModule("builtins") ) );
+        cppy::ptr super_type( builtins.getattr( "super" ) );
+        // Create super args (tuple steals references)
+        cppy::ptr super_args( PyTuple_New(2) );
+        PyTuple_SET_ITEM( super_args.get(), 0, cppy::incref( pyobject_cast( Py_TYPE(m_list.get()) ) ) );
+        PyTuple_SET_ITEM( super_args.get(), 1, cppy::incref( m_list.get() ) );
+
+        // Get and call super method
+        cppy::ptr super( super_type.call( super_args ) );
+        cppy::ptr meth( super.getattr( "sort" ) );
         cppy::ptr res( meth.call(args, kwargs) );
         if( !res )
             return 0;
+
         if( observer_check() )
         {
             cppy::ptr c( prepare_change() );
@@ -822,7 +838,7 @@ private:
         m_obsa = false;
         if( !member() || !atom() )
             return false;
-        m_obsm = member()->has_observers();
+        m_obsm = member()->has_observers( ChangeType::Container );
         m_obsa = atom()->has_observers( member()->name );
         return m_obsm || m_obsa;
     }
@@ -851,12 +867,12 @@ private:
         PyTuple_SET_ITEM( args.get(), 0, change.release() );
         if( m_obsm )
         {
-            if( !member()->notify( atom(), args.get(), 0 ) )
+            if( !member()->notify( atom(), args.get(), 0, ChangeType::Container ) )
                 return false;
         }
         if( m_obsa )
         {
-            if( !atom()->notify( member()->name, args.get(), 0 ) )
+            if( !atom()->notify( member()->name, args.get(), 0, ChangeType::Container ) )
                 return false;
         }
         return true;
