@@ -1,5 +1,5 @@
 /*-----------------------------------------------------------------------------
-| Copyright (c) 2013-2019, Nucleic Development Team.
+| Copyright (c) 2013-2021, Nucleic Development Team.
 |
 | Distributed under the terms of the Modified BSD License.
 |
@@ -101,6 +101,7 @@ enum Mode
     List,
     Set,
     Dict,
+    NonOptional,
     Delegate,
     CallObject,
     CallObject_Object,
@@ -123,27 +124,26 @@ enum Mode
     Bool,
     Int,
     IntPromote,
-    Long,
-    LongPromote,
     Float,
     FloatPromote,
     Bytes,
     BytesPromote,
     Str,
     StrPromote,
-    Unicode,
-    UnicodePromote,
     Tuple,
     List,
     ContainerList,
     Set,
     Dict,
+    OptionalInstance,
     Instance,
+    OptionalTyped,
     Typed,
     Subclass,
     Enum,
     Callable,
     FloatRange,
+    FloatRangePromote,
     Range,
     Coerced,
     Delegate,
