@@ -1,5 +1,5 @@
 /*-----------------------------------------------------------------------------
-| Copyright (c) 2013-2019, Nucleic Development Team.
+| Copyright (c) 2013-2021, Nucleic Development Team.
 |
 | Distributed under the terms of the Modified BSD License.
 |
@@ -7,6 +7,7 @@
 |----------------------------------------------------------------------------*/
 #include <cppy/cppy.h>
 #include "enumtypes.h"
+#include "observer.h"
 #include "packagenaming.h"
 
 #define expand_enum( e ) #e, e
@@ -23,15 +24,11 @@ PyObject* PyPostSetAttr = 0;
 PyObject* PyDefaultValue = 0;
 PyObject* PyValidate = 0;
 PyObject* PyPostValidate = 0;
+PyObject* PyChangeType = 0;
 
 
 namespace {
 
-
-static PyObject* PyIntEnumMeta = 0;
-static PyObject* PyIntEnum = 0;
-
-
 template<typename T> inline bool
 add_long( cppy::ptr& dict_ptr, const char* name, T value )
 {
@@ -50,18 +47,13 @@ add_long( cppy::ptr& dict_ptr, const char* name, T value )
 
 
 inline PyObject*
-make_enum( const char* name, cppy::ptr& dict_ptr )
+make_enum( cppy::ptr& enum_cls, const char* name, cppy::ptr& dict_ptr )
 {
     cppy::ptr pyname( PyUnicode_FromString( name ) );
     if( !pyname )
     {
         return 0;
     }
-    cppy::ptr pybases( PyTuple_Pack( 1, PyIntEnum ) );
-    if( !pybases )
-    {
-        return 0;
-    }
     cppy::ptr pydict( PyDict_Copy( dict_ptr.get() ) );
     if( !pydict )
     {
@@ -72,16 +64,21 @@ make_enum( const char* name, cppy::ptr& dict_ptr )
     {
         return 0;
     }
-    if( PyDict_SetItemString( pydict.get(), "__module__", modname.get() ) != 0 )
+    cppy::ptr kwargs( PyDict_New() );
+    if( !kwargs )
     {
         return 0;
     }
-    cppy::ptr callargs( PyTuple_Pack( 3, pyname.get(), pybases.get(), pydict.get() ) );
+    if( PyDict_SetItemString( kwargs.get(), "module", modname.get() ) != 0 )
+    {
+        return 0;
+    }
+    cppy::ptr callargs( PyTuple_Pack( 2, pyname.get(), pydict.get() ) );
     if( !callargs )
     {
         return 0;
     }
-    cppy::ptr enumclass( PyObject_CallObject( PyIntEnumMeta, callargs.get() ) );
+    cppy::ptr enumclass( enum_cls.call( callargs, kwargs ) );
     if( !enumclass )
     {
         return 0;
@@ -94,18 +91,18 @@ make_enum( const char* name, cppy::ptr& dict_ptr )
 
 bool init_enumtypes()
 {
-    cppy::ptr intenum_mod( PyImport_ImportModule( "atom.intenum" ) );
+    cppy::ptr intenum_mod( PyImport_ImportModule( "enum" ) );
     if( !intenum_mod )
     {
         return false;
     }
-    PyIntEnumMeta = intenum_mod.getattr( "_IntEnumMeta" );
-    if( !PyIntEnumMeta )
+    cppy::ptr enum_cls( intenum_mod.getattr( "IntEnum" ) );
+    if( !enum_cls )
     {
         return false;
     }
-    PyIntEnum = intenum_mod.getattr( "IntEnum" );
-    if( !PyIntEnum )
+    cppy::ptr flag_cls( intenum_mod.getattr( "IntFlag" ) );
+    if( !flag_cls )
     {
         return false;
     }
@@ -129,7 +126,7 @@ bool init_enumtypes()
         add_long( dict_ptr, expand_enum( ObjectMethod ) );
         add_long( dict_ptr, expand_enum( ObjectMethod_Name ) );
         add_long( dict_ptr, expand_enum( MemberMethod_Object ) );
-        PyGetAttr = make_enum( "GetAttr", dict_ptr );
+        PyGetAttr = make_enum( enum_cls, "GetAttr", dict_ptr );
         if( !PyGetAttr )
         {
                return false;
@@ -156,7 +153,7 @@ bool init_enumtypes()
         add_long( dict_ptr, expand_enum( ObjectMethod_Value ) );
         add_long( dict_ptr, expand_enum( ObjectMethod_NameValue ) );
         add_long( dict_ptr, expand_enum( MemberMethod_ObjectValue ) );
-        PySetAttr = make_enum( "SetAttr", dict_ptr );
+        PySetAttr = make_enum( enum_cls, "SetAttr", dict_ptr );
         if( !PySetAttr )
         {
                return false;
@@ -178,7 +175,7 @@ bool init_enumtypes()
         add_long( dict_ptr, expand_enum( Signal ) );
         add_long( dict_ptr, expand_enum( Delegate ) );
         add_long( dict_ptr, expand_enum( Property ) );
-        PyDelAttr = make_enum( "DelAttr", dict_ptr );
+        PyDelAttr = make_enum( enum_cls, "DelAttr", dict_ptr );
         if( !PyDelAttr )
         {
                return false;
@@ -197,7 +194,7 @@ bool init_enumtypes()
         add_long( dict_ptr, expand_enum( ObjectMethod_Value ) );
         add_long( dict_ptr, expand_enum( ObjectMethod_NameValue ) );
         add_long( dict_ptr, expand_enum( MemberMethod_ObjectValue ) );
-        PyPostGetAttr = make_enum( "PostGetAttr", dict_ptr );
+        PyPostGetAttr = make_enum( enum_cls, "PostGetAttr", dict_ptr );
         if( !PyPostGetAttr )
         {
                return false;
@@ -216,7 +213,7 @@ bool init_enumtypes()
         add_long( dict_ptr, expand_enum( ObjectMethod_OldNew ) );
         add_long( dict_ptr, expand_enum( ObjectMethod_NameOldNew ) );
         add_long( dict_ptr, expand_enum( MemberMethod_ObjectOldNew ) );
-        PyPostSetAttr = make_enum( "PostSetAttr", dict_ptr );
+        PyPostSetAttr = make_enum( enum_cls, "PostSetAttr", dict_ptr );
         if( !PyPostSetAttr )
         {
                return false;
@@ -235,6 +232,7 @@ bool init_enumtypes()
         add_long( dict_ptr, expand_enum( List ) );
         add_long( dict_ptr, expand_enum( Set ) );
         add_long( dict_ptr, expand_enum( Dict ) );
+        add_long( dict_ptr, expand_enum( NonOptional ) );
         add_long( dict_ptr, expand_enum( Delegate ) );
         add_long( dict_ptr, expand_enum( CallObject ) );
         add_long( dict_ptr, expand_enum( CallObject_Object ) );
@@ -242,7 +240,7 @@ bool init_enumtypes()
         add_long( dict_ptr, expand_enum( ObjectMethod ) );
         add_long( dict_ptr, expand_enum( ObjectMethod_Name ) );
         add_long( dict_ptr, expand_enum( MemberMethod_Object ) );
-        PyDefaultValue = make_enum( "DefaultValue", dict_ptr );
+        PyDefaultValue = make_enum( enum_cls, "DefaultValue", dict_ptr );
         if( !PyDefaultValue )
         {
                return false;
@@ -260,34 +258,33 @@ bool init_enumtypes()
         add_long( dict_ptr, expand_enum( Bool ) );
         add_long( dict_ptr, expand_enum( Int ) );
         add_long( dict_ptr, expand_enum( IntPromote ) );
-        add_long( dict_ptr, expand_enum( Long ) );
-        add_long( dict_ptr, expand_enum( LongPromote ) );
         add_long( dict_ptr, expand_enum( Float ) );
         add_long( dict_ptr, expand_enum( FloatPromote ) );
         add_long( dict_ptr, expand_enum( Bytes ) );
         add_long( dict_ptr, expand_enum( BytesPromote ) );
         add_long( dict_ptr, expand_enum( Str ) );
         add_long( dict_ptr, expand_enum( StrPromote ) );
-        add_long( dict_ptr, expand_enum( Unicode ) );
-        add_long( dict_ptr, expand_enum( UnicodePromote ) );
         add_long( dict_ptr, expand_enum( Tuple ) );
         add_long( dict_ptr, expand_enum( List ) );
         add_long( dict_ptr, expand_enum( ContainerList ) );
         add_long( dict_ptr, expand_enum( Set ) );
         add_long( dict_ptr, expand_enum( Dict ) );
+        add_long( dict_ptr, expand_enum( OptionalInstance ) );
         add_long( dict_ptr, expand_enum( Instance ) );
+        add_long( dict_ptr, expand_enum( OptionalTyped ) );
         add_long( dict_ptr, expand_enum( Typed ) );
         add_long( dict_ptr, expand_enum( Subclass ) );
         add_long( dict_ptr, expand_enum( Enum ) );
         add_long( dict_ptr, expand_enum( Callable ) );
         add_long( dict_ptr, expand_enum( FloatRange ) );
+        add_long( dict_ptr, expand_enum( FloatRangePromote ) );
         add_long( dict_ptr, expand_enum( Range ) );
         add_long( dict_ptr, expand_enum( Coerced ) );
         add_long( dict_ptr, expand_enum( Delegate ) );
         add_long( dict_ptr, expand_enum( ObjectMethod_OldNew ) );
         add_long( dict_ptr, expand_enum( ObjectMethod_NameOldNew ) );
         add_long( dict_ptr, expand_enum( MemberMethod_ObjectOldNew ) );
-        PyValidate = make_enum( "Validate", dict_ptr );
+        PyValidate = make_enum( enum_cls, "Validate", dict_ptr );
         if( !PyValidate )
         {
                return false;
@@ -306,13 +303,32 @@ bool init_enumtypes()
         add_long( dict_ptr, expand_enum( ObjectMethod_OldNew ) );
         add_long( dict_ptr, expand_enum( ObjectMethod_NameOldNew ) );
         add_long( dict_ptr, expand_enum( MemberMethod_ObjectOldNew ) );
-        PyPostValidate = make_enum( "PostValidate", dict_ptr );
+        PyPostValidate = make_enum( enum_cls, "PostValidate", dict_ptr );
         if( !PyPostValidate )
         {
             return false;
         }
     }
-
+    {
+        using namespace ChangeType;
+        cppy::ptr dict_ptr( PyDict_New() );
+        if( !dict_ptr )
+        {
+            return false;  // LCOV_EXCL_LINE
+        }
+        add_long( dict_ptr, "CREATE", Create );
+        add_long( dict_ptr, "UPDATE", Update );
+        add_long( dict_ptr, "DELETE", Delete );
+        add_long( dict_ptr, "EVENT",  Event );
+        add_long( dict_ptr, "PROPERTY", Property );
+        add_long( dict_ptr, "CONTAINER", Container );
+        add_long( dict_ptr, "ANY", Any );
+        PyChangeType = make_enum( flag_cls, "ChangeType", dict_ptr );
+        if( !PyChangeType )
+        {
+            return false;
+        }
+    }
     return true;
 }
 
