@@ -1,12 +1,15 @@
 /*-----------------------------------------------------------------------------
-| Copyright (c) 2013, Nucleic Development Team.
+| Copyright (c) 2013-2019, Nucleic Development Team.
 |
 | Distributed under the terms of the Modified BSD License.
 |
-| The full license is in the file COPYING.txt, distributed with this software.
+| The full license is in the file LICENSE, distributed with this software.
 |----------------------------------------------------------------------------*/
 #pragma once
+#include <Python.h>
 
+namespace atom
+{
 
 struct ModifyTask
 {
@@ -30,6 +33,16 @@ public:
 
     ~ModifyGuard()
     {
+        // If an exception occurred we store it and restore it after the
+        // modification have been done as otherwise it can (Python 3) cause
+        // boolean tests (PyObject_IsTrue) to fail for wrong reasons.
+        bool exception_set = false;
+        PyObject *type, *value, *traceback;
+        if( PyErr_Occurred() ){
+            PyErr_Fetch(&type, &value, &traceback);
+            exception_set = true;
+        }
+
         if( m_owner.get_modify_guard() == this )
         {
             m_owner.set_modify_guard( 0 );
@@ -41,6 +54,10 @@ public:
                 delete *it;
             }
         }
+
+        // Restore previous exception if one was set.
+        if( exception_set )
+            PyErr_Restore(type, value, traceback);
     }
 
     void add_task( ModifyTask* task ) { m_tasks.push_back( task ); }
@@ -51,3 +68,5 @@ private:
     std::vector<ModifyTask*> m_tasks;
 
 };
+
+}  // namespace atom
