@@ -1,5 +1,5 @@
 /*-----------------------------------------------------------------------------
-| Copyright (c) 2013-2019, Nucleic Development Team.
+| Copyright (c) 2013-2021, Nucleic Development Team.
 |
 | Distributed under the terms of the Modified BSD License.
 |
@@ -26,15 +26,15 @@ bool validate_type_tuple_types( PyObject* type_tuple_types )
 {
     if( PyTuple_Check( type_tuple_types ) )
     {
-        int len = PySequence_Size( type_tuple_types );
-        for( int i = 0; i < len; i++ )
+        Py_ssize_t len = PySequence_Size( type_tuple_types );
+        for( Py_ssize_t i = 0; i < len; i++ )
         {
             if( !PyType_Check( PyTuple_GET_ITEM( type_tuple_types, i ) ) )
             {
                 PyErr_Format(
                     PyExc_TypeError,
                     "Expected type or tuple of types. Got a tuple containing an instance of `%s` instead.",
-                    PyTuple_GET_ITEM( type_tuple_types, i )->ob_type->tp_name
+                    Py_TYPE( PyTuple_GET_ITEM( type_tuple_types, i ) )->tp_name
                 );
                 return false;
             }
@@ -93,11 +93,13 @@ Member::check_context( Validate::Mode mode, PyObject* context )
             }
             break;
         }
+        case Validate::OptionalInstance:
         case Validate::Instance:
         case Validate::Subclass:
         {
             return validate_type_tuple_types( context );
         }
+        case Validate::OptionalTyped:
         case Validate::Typed:
             if( !PyType_Check( context ) )
             {
@@ -233,8 +235,8 @@ std::string name_from_type_tuple_types( PyObject* type_tuple_types )
     else
     {
         ostr << "(";
-        int len = PySequence_Size( type_tuple_types );
-        for( int i = 0; i < len; i++ )
+        Py_ssize_t len = PySequence_Size( type_tuple_types );
+        for( Py_ssize_t i = 0; i < len; i++ )
         {
             PyTypeObject* type = pytype_cast( PyTuple_GET_ITEM( type_tuple_types, i ) );
             ostr << type->tp_name;
@@ -256,9 +258,9 @@ validate_type_fail( Member* member, CAtom* atom, PyObject* newvalue, const char*
         "The '%s' member on the '%s' object must be of type '%s'. "
         "Got object of type '%s' instead.",
         PyUnicode_AsUTF8( member->name ),
-        pyobject_cast( atom )->ob_type->tp_name,
+        Py_TYPE( pyobject_cast( atom ) )->tp_name,
         type,
-        newvalue->ob_type->tp_name
+        Py_TYPE( newvalue )->tp_name
     );
     return 0;
 }
@@ -570,10 +572,8 @@ dict_handler( Member* member, CAtom* atom, PyObject* oldvalue, PyObject* newvalu
 
 
 PyObject*
-instance_handler( Member* member, CAtom* atom, PyObject* oldvalue, PyObject* newvalue )
+non_optional_instance_handler( Member* member, CAtom* atom, PyObject* oldvalue, PyObject* newvalue )
 {
-    if( newvalue == Py_None )
-        return cppy::incref( newvalue );
     int res = PyObject_IsInstance( newvalue, member->validate_context );
     if( res < 0 )
         return 0;
@@ -584,10 +584,17 @@ instance_handler( Member* member, CAtom* atom, PyObject* oldvalue, PyObject* new
 
 
 PyObject*
-typed_handler( Member* member, CAtom* atom, PyObject* oldvalue, PyObject* newvalue )
+instance_handler( Member* member, CAtom* atom, PyObject* oldvalue, PyObject* newvalue )
 {
     if( newvalue == Py_None )
         return cppy::incref( newvalue );
+    return non_optional_instance_handler( member, atom, oldvalue, newvalue );
+}
+
+
+PyObject*
+non_optional_typed_handler( Member* member, CAtom* atom, PyObject* oldvalue, PyObject* newvalue )
+{
     PyTypeObject* type = pytype_cast( member->validate_context );
     if( PyObject_TypeCheck( newvalue, type ) )
         return cppy::incref( newvalue );
@@ -595,6 +602,15 @@ typed_handler( Member* member, CAtom* atom, PyObject* oldvalue, PyObject* newval
 }
 
 
+PyObject*
+typed_handler( Member* member, CAtom* atom, PyObject* oldvalue, PyObject* newvalue )
+{
+    if( newvalue == Py_None )
+        return cppy::incref( newvalue );
+    return non_optional_typed_handler( member, atom, oldvalue, newvalue );
+}
+
+
 PyObject*
 subclass_handler( Member* member, CAtom* atom, PyObject* oldvalue, PyObject* newvalue )
 {
@@ -606,9 +622,9 @@ subclass_handler( Member* member, CAtom* atom, PyObject* oldvalue, PyObject* new
             "The '%s' member on the '%s' object must be a subclass of '%s'. "
             "Got instance of '%s' instead.",
             PyUnicode_AsUTF8( member->name ),
-            pyobject_cast( atom )->ob_type->tp_name,
+            Py_TYPE( pyobject_cast( atom ) )->tp_name,
             name_from_type_tuple_types( member->validate_context ).c_str(),
-            newvalue->ob_type->tp_name
+            Py_TYPE( newvalue )->tp_name
         );
         return 0;
     }
@@ -627,7 +643,7 @@ subclass_handler( Member* member, CAtom* atom, PyObject* oldvalue, PyObject* new
             "The '%s' member on the '%s' object must be a subclass of '%s'. "
             "Got class '%s' instead.",
             PyUnicode_AsUTF8( member->name ),
-            pyobject_cast( atom )->ob_type->tp_name,
+            Py_TYPE( pyobject_cast( atom ) )->tp_name,
             name_from_type_tuple_types( member->validate_context ).c_str(),
             type->tp_name
         );
@@ -682,6 +698,23 @@ float_range_handler( Member* member, CAtom* atom, PyObject* oldvalue, PyObject*
 }
 
 
+PyObject*
+float_range_promote_handler( Member* member, CAtom* atom, PyObject* oldvalue, PyObject* newvalue )
+{
+    if( PyFloat_Check( newvalue ) )
+        return float_range_handler( member, atom, oldvalue, newvalue );
+    if( PyLong_Check( newvalue ) )
+    {
+        double value = PyLong_AsDouble( newvalue );
+        if( value == -1.0 && PyErr_Occurred() )
+            return 0;
+        cppy::ptr convertedvalue( PyFloat_FromDouble( value ) );
+        return float_range_handler( member, atom, oldvalue, convertedvalue.get() );
+    }
+    return validate_type_fail( member, atom, newvalue, "float" );
+}
+
+
 PyObject*
 range_handler( Member* member, CAtom* atom, PyObject* oldvalue, PyObject* newvalue )
 {
@@ -798,27 +831,26 @@ handlers[] = {
     bool_handler,
     long_handler,
     long_promote_handler,
-    long_handler,
-    long_promote_handler,
     float_handler,
     float_promote_handler,
     bytes_handler,
     bytes_promote_handler,
     str_handler,
     str_promote_handler,
-    str_handler,
-    str_promote_handler,
     tuple_handler,
     list_handler,
     container_list_handler,
     set_handler,
     dict_handler,
     instance_handler,
+    non_optional_instance_handler,
     typed_handler,
+    non_optional_typed_handler,
     subclass_handler,
     enum_handler,
     callable_handler,
     float_range_handler,
+    float_range_promote_handler,
     range_handler,
     coerced_handler,
     delegate_handler,
