@@ -1,10 +1,10 @@
-#------------------------------------------------------------------------------
-# Copyright (c) 2013-2017, Nucleic Development Team.
+# --------------------------------------------------------------------------------------
+# Copyright (c) 2013-2021, Nucleic Development Team.
 #
 # Distributed under the terms of the Modified BSD License.
 #
 # The full license is in the file LICENSE, distributed with this software.
-#------------------------------------------------------------------------------
+# --------------------------------------------------------------------------------------
 """ Demonstrate all the ways to initialize a value
 
 1. Pass the value directly
@@ -14,44 +14,38 @@
 5. Use a _default_* static method
 
 """
-from __future__ import (division, unicode_literals, print_function,
-                        absolute_import)
-
 import sys
 
-from atom.api import Atom, Unicode, Int
+from atom.api import Atom, Int, Str
 
 
 def get_mother():
-    return 'Maude ' + get_last_name()
+    return "Maude " + get_last_name()
 
 
 def get_last_name():
-    """ Return a last name based on the system byteorder.
-
-    """
+    """Return a last name based on the system byteorder."""
     return sys.byteorder.capitalize()
 
 
 class Person(Atom):
-    """ A simple class representing a person object.
+    """A simple class representing a person object."""
 
-    """
-    first_name = Unicode('Bob')
+    first_name = Str("Bob")
 
     age = Int(default=40)
 
-    address = Unicode()
+    address = Str()
 
-    mother = Unicode(factory=get_mother)
+    mother = Str(factory=get_mother)
 
-    last_name = Unicode()
+    last_name = Str()
 
     def _default_last_name(self):
         return get_last_name()
 
 
-if __name__ == '__main__':
-    bob = Person(address='101 Main')
+if __name__ == "__main__":
+    bob = Person(address="101 Main")
     print((bob.first_name, bob.last_name, bob.age))
     print(bob.mother)
