@@ -1,19 +1,21 @@
-#------------------------------------------------------------------------------
-# Copyright (c) 2018, Nucleic Development Team.
+# --------------------------------------------------------------------------------------
+# Copyright (c) 2018-2021, Nucleic Development Team.
 #
 # Distributed under the terms of the Modified BSD License.
 #
 # The full license is in the file LICENSE, distributed with this software.
-#------------------------------------------------------------------------------
+# --------------------------------------------------------------------------------------
 """Test the signal connectors.
 
 """
 import gc
 import operator
 import sys
+from collections import Counter
 
 import pytest
-from atom.api import Atom, Signal, Int
+
+from atom.api import Atom, Signal
 
 
 def test_signalconnector_lifecycle():
@@ -23,6 +25,7 @@ def test_signalconnector_lifecycle():
     deallocate some.
 
     """
+
     class SignalAtom(Atom):
         s = Signal()
 
@@ -34,13 +37,17 @@ def test_signalconnector_lifecycle():
 
     atom = SignalAtom()
     sc = atom.s
-    assert gc.get_referents(sc) == [SignalAtom.s, atom]
+    # Under Python 3.9+ heap allocated type instance keep a reference to the
+    # type
+    referents = [SignalAtom.s, atom]
+    if sys.version_info >= (3, 9):
+        referents.append(type(sc))
+    assert Counter(gc.get_referents(sc)) == Counter(referents)
 
 
 def test_signalconnector_cmp():
-    """Test comparing event binders.
+    """Test comparing event binders."""
 
-    """
     class EventAtom(Atom):
         s1 = Signal()
         s2 = Signal()
@@ -51,6 +58,6 @@ def test_signalconnector_cmp():
     assert not a.s1 == 1
 
     if sys.version_info >= (3,):
-        for op in ('lt', 'le', 'gt', 'ge'):
+        for op in ("lt", "le", "gt", "ge"):
             with pytest.raises(TypeError):
                 getattr(operator, op)(a.s1, 1)
